
#include <wx/wx.h>
#include <wx/html/htmlwin.h>

#include "afhelp.h"
#include "stock_close.xpm"
#include "stock_left_arrow.xpm"
#include "stock_right_arrow.xpm"
#include "aficon.xpm"

enum { AFHELP_CLOSE, AFHELP_BACK, AFHELP_FORWARD };

BEGIN_EVENT_TABLE( AFHelp, wxFrame )
	EVT_TOOL( AFHELP_BACK, AFHelp::OnTool )
	EVT_TOOL( AFHELP_FORWARD, AFHelp::OnTool )
	EVT_TOOL( AFHELP_CLOSE, AFHelp::OnTool )
	
	EVT_CLOSE( AFHelp::OnCloseFrame )
END_EVENT_TABLE();

AFHelp::AFHelp(const wxString &zipfile)
	: wxFrame(NULL, -1, "Xpmedit Help", wxDefaultPosition, wxSize(600, 600))
{

#ifdef __WXMSW__
	SetIcon( wxIcon("aficon") );
#else
	SetIcon( wxIcon( aficon_xpm ) );
#endif

	CreateStatusBar(1);

	mZipFilePath = zipfile;
		
	mToolBar = CreateToolBar();
	mToolBar->AddTool( AFHELP_CLOSE, wxBitmap( stock_close_xpm ), "Close Help" );
	mToolBar->AddTool( AFHELP_BACK, wxBitmap( stock_left_arrow_xpm ), "Go Back" );
	mToolBar->AddTool( AFHELP_FORWARD, wxBitmap( stock_right_arrow_xpm ), "Go Forward" );
    mToolBar->SetToolBitmapSize(wxSize(24, 24));	
	mToolBar->Realize();

    mHtml = new wxHtmlWindow( this );
    mHtml->SetRelatedFrame(this, "Xpmedit Help : %s");
    mHtml->SetRelatedStatusBar(0);
    mHtml->LoadPage(mZipFilePath);
}

AFHelp::~AFHelp()
{
	/* nothing to do here */
}

void AFHelp::OnTool(wxCommandEvent &evt)
{
	switch(evt.GetId())
	{
	case AFHELP_CLOSE:
		Hide();
		break;
	case AFHELP_BACK:
		mHtml->HistoryBack();
		break;
	case AFHELP_FORWARD:
		mHtml->HistoryForward();
	default:
		break;
	}
}

void AFHelp::OnCloseFrame(wxCloseEvent &evt)
{
	Hide();
	evt.Veto();
}
