#ifndef AFVIEW_H
#define AFVIEW_H

#include <wx/wx.h>
#include "painter.h"
#include "crect.h"

class wxConfig;

class ViewInfo
{
public:
	void WriteToConfig(wxConfig *cfg);
	void ReadFromConfig(wxConfig *cfg);

	bool showgrid;
	int xoff;
	int yoff;
	int psz;
	int pixwidth;
	int pixheight;
};

class AFFrame;

class AFView : public wxWindow
{
public:
	AFView(AFFrame *frm);
	virtual ~AFView();
	
	void GetViewInfo(ViewInfo *info);
	void SetViewInfo(ViewInfo &info, bool redraw = true);

	void SetPrinting(bool b);
	bool IsPrinting();

	bool IsDirty() { return bDirty; }
	bool SetDirty(bool b) { bDirty = b; }

	void ZoomIn();
	void ZoomOut();

	void RedrawAll();
	void Draw(wxDC &dc);
	void DrawGrid(Painter &gfx);

	void OnButtonDown(wxMouseEvent &evt);
	void OnButtonUp(wxMouseEvent &evt);
	void OnMouseMove(wxMouseEvent &evt);
	void OnKeyEvent(wxKeyEvent &evt);
	void OnSize(wxSizeEvent &evt);
	void OnPaint(wxPaintEvent &evt);


	wxImage &GetImage();

	void Clear(int w, int h);
	void Resize(const wxSize &sz);
	bool LoadFile(const wxString &name, int type = wxBITMAP_TYPE_XPM);
	bool SaveFile(const wxString &name, int type = wxBITMAP_TYPE_XPM);

private:
	void DrawPix(int cx, int cy, const wxColour &c);
	void ChangeColor(wxColour *color);
	wxColour mColor1, mColor2;

	bool bDirty;

	wxImage mImage;

	ViewInfo mVI;
			
	wxCursor mCursorDefault;
	wxCursor mCursorDelete;
	wxCursor mCursorCopy;
	wxCursor mCursorInsert;
	
	wxColour mMaskColor;
	wxColour mSelectColor;
	wxColour mForeground;
	wxColour mBackground;

	wxClientDC *mClientDC;
	Painter *mClientPainter;

	wxBitmap *mBuffer;
	AFFrame *mFrame;

	bool bIsPrinting;

	void GetClientRect(CRect &rct);
	
DECLARE_EVENT_TABLE()
};

#endif
