
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <wx/wx.h>
#include <wx/scrolbar.h>
#include <wx/print.h>
#include <wx/printdlg.h>
#include <wx/accel.h>
#include <wx/image.h>
#include <wx/fs_zip.h>
#include <wx/html/htmlwin.h>
#include <wx/config.h>

#include "painter.h"
#include "afhelp.h"
#include "afwx.h"
#include "afview.h"
#include "tokenizer.h"

#include "stock_new.xpm"
#include "stock_open.xpm"
#include "stock_save.xpm"
#include "stock_save_as.xpm"
#include "stock_trash_full.xpm"
#include "stock_print.xpm"
#include "stock_convert.xpm"
#include "stock_undo.xpm"
#include "stock_redo.xpm"
#include "stock_button_cancel.xpm"
#include "stock_zoom_in.xpm"
#include "stock_zoom_out.xpm"
#include "stock_zoom_fit.xpm"
#include "stock_jump_to.xpm"
#include "stock_refresh.xpm"
#include "stock_index.xpm"
#include "stock_help.xpm"
#include "aficon.xpm"


/* application global variables */

static AFFrame *app_frame = NULL;
static wxPrintData *app_printData = NULL;
static wxPageSetupData *app_pageSetupData = NULL;

static wxArrayString app_args;

wxConfig *app_config = NULL;

/* AFFrame implementation */

IMPLEMENT_APP(AFApp)

AFApp::AFApp()
{
	/* nothing to do */
}

bool AFApp::OnInit()
{
	for (int i=0;i<argc;i++)
		app_args.Add(argv[i]);

#ifdef MESSAGES
	printf("Xpmedit (WX): Starting up...\n");
	printf("Framework by Aron Dobos\n");
	printf("Command line args: %d\n", app_args.Count());
	for (int k=0;k<app_args.Count();k++)
		printf("argv[%d]: '%s'\n", k, app_args[k].c_str());
	
#endif

	app_config = new wxConfig("Xpmedit", "AF_Applications");


	/* needed for the html help viewer */
	wxImage::AddHandler(new wxBMPHandler);
    wxImage::AddHandler(new wxPNGHandler);
    wxImage::AddHandler(new wxJPEGHandler);
	wxImage::AddHandler(new wxXPMHandler);
    wxFileSystem::AddHandler(new wxZipFSHandler);

	app_printData = new wxPrintData;
	app_printData->SetPaperId(wxPAPER_LETTER);
	app_printData->SetQuality(wxPRINT_QUALITY_HIGH);

	app_pageSetupData = new wxPageSetupDialogData;

	app_frame = new AFFrame;
	app_frame->Centre(wxBOTH);
	app_frame->Show(true);

	SetTopWindow(app_frame);

	if ((int)app_args.Count() > 1)
		app_frame->Load(app_args[1]);
	
	
	return true;
}

int AFApp::OnExit()
{	
	// delete the app config
	// (it automatically writes the configuration)
	delete app_config;

	delete app_printData;
	delete app_pageSetupData;
	
#ifdef MESSAGES
	printf("Xpmedit: Shutdown complete.\n");
#endif
	
	return 0;
}

enum{  
		ID_NEW, ID_OPEN, ID_SAVE, ID_SAVEAS, ID_CLOSEFILE,
		ID_PRINT, ID_PRINT_SETUP, ID_PAGE_SETUP, ID_PREVIEW,
		ID_RECENT_FILES, ID_EXIT,

		ID_RESIZE,
		
		ID_ABOUT, 
		ID_HELP,
		
		// up to 100 recent items can be accommodated
		ID_RECENT = 500,
		ID_RECENT_LAST = 600,

};

BEGIN_EVENT_TABLE(AFFrame, wxFrame)
	EVT_MENU( ID_NEW,                    AFFrame::OnDocumentCommand )
	EVT_MENU( ID_RESIZE,                 AFFrame::OnDocumentCommand )
	EVT_MENU( ID_OPEN,                   AFFrame::OnDocumentCommand )
	EVT_MENU( ID_SAVE,                   AFFrame::OnDocumentCommand )
	EVT_MENU( ID_SAVEAS,                 AFFrame::OnDocumentCommand )
	EVT_MENU( ID_CLOSEFILE,              AFFrame::OnDocumentCommand )

	EVT_MENU( ID_PRINT,                  AFFrame::OnPrintCommand )
	EVT_MENU( ID_PRINT_SETUP,            AFFrame::OnPrintCommand )
	EVT_MENU( ID_PAGE_SETUP,             AFFrame::OnPrintCommand )
	EVT_MENU( ID_PREVIEW,                AFFrame::OnPrintCommand )
	
	EVT_MENU( ID_EXIT,                   AFFrame::OnExit )

	EVT_MENU( ID_ABOUT, AFFrame::OnAbout )
	EVT_MENU( ID_HELP, AFFrame::OnHelp )
	
	EVT_CLOSE( AFFrame::OnCloseFrame )

	// For recent file menu
	EVT_MENU( ID_RECENT+0, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+1, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+2, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+3, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+4, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+5, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+6, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+7, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+8, AFFrame::OnRecent)
	EVT_MENU( ID_RECENT+9, AFFrame::OnRecent) // this must go all the way to MAX_RECENT-1
	
END_EVENT_TABLE()

AFFrame::AFFrame()
	: wxFrame(NULL, -1, "Xpmedit", wxDefaultPosition, wxSize(700, 600))
{
	mFilename = "";
	
	mRecentCount = 0;

	wxString helpfile = wxPathOnly(app_args[0]) + "/xpmedithelp.zip#zip:index.html";
	mHelpView = new AFHelp(helpfile);


#ifdef __WXMSW__
	SetIcon( wxIcon("aficon") );
#else
	SetIcon( wxIcon( aicon_xpm ) );
#endif

	mRecentMenu = new wxMenu;

	mFileMenu = new wxMenu;
	mFileMenu->Append(ID_NEW, "New\tCtrl-N");
	mFileMenu->Append(ID_RESIZE, "Resize\tCtrl-R");
	mFileMenu->AppendSeparator();
	mFileMenu->Append(ID_OPEN, "Open...\tCtrl-O");
	mFileMenu->Append(ID_SAVE, "Save\tCtrl-S");
	mFileMenu->Append(ID_SAVEAS, "Save As...");
	mFileMenu->AppendSeparator();
	mFileMenu->Append(ID_CLOSEFILE, "Close");
	mFileMenu->AppendSeparator();
	/*
	mFileMenu->Append(ID_PRINT, "Print...");
	mFileMenu->Append(ID_PRINT_SETUP, "Print Setup...");
	mFileMenu->Append(ID_PAGE_SETUP, "Page Setup...");
	mFileMenu->Append(ID_PREVIEW, "Print Preview");
	mFileMenu->AppendSeparator();*/
	mFileMenu->Append(ID_RECENT_FILES, "Recent Files", mRecentMenu);
	mFileMenu->AppendSeparator();
	mFileMenu->Append(ID_EXIT, "Exit\tCtrl-W");


	wxMenu *help_menu = new wxMenu;
	help_menu->Append(ID_ABOUT, "About Xpmedit");
	help_menu->AppendSeparator();
	help_menu->Append(ID_HELP, "Help Contents\tF1");
	
	wxMenuBar *menubar = new wxMenuBar;
	menubar->Append(mFileMenu, "File");
	menubar->Append(help_menu, "Help");
	
	SetMenuBar( menubar );
	
	mToolBar = CreateToolBar();

	mToolBar->AddTool( ID_NEW, wxBitmap( stock_new_xpm ), "Create a new document" );
	mToolBar->AddTool( ID_OPEN, wxBitmap( stock_open_xpm ), "Open an existing file" );
	mToolBar->AddTool( ID_SAVE, wxBitmap( stock_save_xpm ), "Save changes");
	mToolBar->AddTool( ID_SAVEAS, wxBitmap( stock_save_as_xpm ), "Save to a different file" );
	mToolBar->AddTool( ID_HELP, wxBitmap( stock_help_xpm ), "Help contents" );

    mToolBar->SetToolBitmapSize(wxSize(24, 24));	
	mToolBar->Realize();

	mView = new AFView(this);
	
	
	SetSize(670, 600);
	
	mView->SetFocus();
	
	
	long ct = 0;
	if (app_config->Read("RecentCount", &ct))
		mRecentCount = (int)ct;

	if (mRecentCount > MAX_RECENT)
		mRecentCount = MAX_RECENT;

	for (int i=0;i<mRecentCount;i++)
	{
		wxString key;
		key.Printf("RecentFile_%d", i);
		wxString fn;
		if (app_config->Read(key, &fn))
			mRecentFiles[i] = fn;
	}

	UpdateRecentMenu();	
	UpdateControlStatus();
	UpdateTitlebar();

	mView->Clear(24, 24);
	mView->RedrawAll();
}

AFFrame::~AFFrame()
{
	// save the recent file settings

	long ct = (long)mRecentCount;
	app_config->Write("RecentCount", ct);
	for (int i=0;i<mRecentCount;i++)
	{
		wxString key;
		key.Printf("RecentFile_%d", i);
		app_config->Write(key, mRecentFiles[i]);
	}

	mHelpView->Destroy();

#ifdef MESSSAGES
	printf("~AFFrame: destructing the main window...\n");
#endif
}

void AFFrame::UpdateRecentMenu()
{
	int i;
	for (i=0;i<MAX_RECENT;i++)
	{
		if (mRecentMenu->FindItem(ID_RECENT+i) != NULL)
			mRecentMenu->Destroy(ID_RECENT+i);
	}

	for (i=0;i<mRecentCount;i++)
	{
		wxString name;
		name.Printf("%d  %s", i+1, mRecentFiles[i].c_str());
		mRecentMenu->Append(ID_RECENT+i, name);
	}
}

void AFFrame::OnRecent(wxCommandEvent &evt)
{
	int id = evt.GetId() - ID_RECENT;
	if (id < 0 || id >= MAX_RECENT)
		return;

	if (!CloseDocument())
		return;

	wxString fn = mRecentFiles[id];
	if (fn != "")
	{
		//wxMessageBox("loading recent file " + fn);
		Load(fn);
	}
}

void AFFrame::AddRecent(const wxString &fn)
{
	int i;
	int index = -1;
	// find the file in the recent list
	for (i=0;i<mRecentCount;i++)
	{
		if (fn == mRecentFiles[i])
		{
			index = i;
			break;
		}
	}

	if (index >= 0)
	{
		// bring this file to the front of the
		// recent file list

		for (i=index;i>0;i--)
			mRecentFiles[i] = mRecentFiles[i-1];
	}
	else // not found in recent list
	{
		// add this to the front of the recent list
		// and increment the recent count if its 
		// less than MAX_RECENT

		for (i=MAX_RECENT-1;i>0;i--)
			mRecentFiles[i] = mRecentFiles[i-1];

		if (mRecentCount < MAX_RECENT)
			mRecentCount++;
	}
	
	mRecentFiles[0] = fn;
	UpdateRecentMenu();
}

void AFFrame::RemoveRecent(const wxString &fn)
{
	int i;
	int index = -1;
	// find the file in the recent list
	for (i=0;i<mRecentCount;i++)
	{
		if (fn == mRecentFiles[i])
		{
			index = i;
			break;
		}
	}

	if (index >= 0)
	{
		for (i=index;i<MAX_RECENT-1;i++)
			mRecentFiles[i] = mRecentFiles[i+1];

		mRecentCount--;
		UpdateRecentMenu();
	}
}

void AFFrame::ShowHelp()
{
	mHelpView->Show();
}

void AFFrame::OnCloseFrame( wxCloseEvent &evt )
{
	if (evt.CanVeto() && (!CloseDocument()))
	{
		evt.Veto();
		return;
	}
	
	Destroy();
}

wxSize AFFrame::GetNewSize(int w, int h)
{
	wxString initial;
	initial.Printf("%dx%d", w, h);
	wxString sz = wxGetTextFromUser("Enter the size for the image (width x height):", "Xpmedit", initial);

	if (sz != wxEmptyString)
	{
		Tokenizer tokens(sz, " \tx,:;");

		wxString sx, sy;
		tokens.get(sx);
		tokens.get(sy);

		w = atoi(sx.c_str());
		h = atoi(sy.c_str());

		if (w > 1 && h > 1 && w < MAXWIDTH && h < MAXHEIGHT)
			return wxSize(w,h);
	}
	
	return wxSize(-1,-1);
}

void AFFrame::New()
{
	if (!CloseDocument())
		return;

	// INITIALIZE NEW DOCUMENT HERE

	mFilename = "";

	wxSize sz = GetNewSize();
	int w = sz.GetWidth();
	int h = sz.GetHeight();
	
	if ( w < 0 || h < 0)
	{
		wxMessageBox("Invalid size entered.  Using default size.", "Xpmedit", wxICON_ERROR|wxOK);
		w = h = 24;
	}
	
	mView->Clear(w, h);

	mView->RedrawAll();
	
	UpdateTitlebar();
	UpdateControlStatus();
}


void AFFrame::Load(const wxString &fn)
{
	mFilename = fn;


	wxFileName wfn(fn);

	int type = wxBITMAP_TYPE_XPM;
	
	if (wfn.GetExt().CmpNoCase("bmp")==0)
		type = wxBITMAP_TYPE_BMP;
	else if (wfn.GetExt().CmpNoCase("png")==0)
		type = wxBITMAP_TYPE_PNG;

	// READ DATA HERE
	if (!mView->LoadFile( fn, type ))
	{
		wxMessageBox("Error loading xpm image.", "Xpmedit", wxICON_ERROR|wxOK);
		mFilename = "";
	}
	else
	{
		AddRecent( fn );
	}

	mView->RedrawAll();

	UpdateTitlebar();
	UpdateControlStatus();
}

void AFFrame::Open()
{
	if (!CloseDocument())
		return;

	wxString initial_dir = "";
	
	wxFileDialog fdlg(this, "Open File", initial_dir, "", "XPM Files(*.xpm)|*.xpm|BMP Files(*.bmp)|*.bmp|PNG Files(*.png)|*.png", wxOPEN);
	if (fdlg.ShowModal() == wxID_OK)
		Load( fdlg.GetPath() );
}

void AFFrame::Save()
{
	if (mFilename == "")
	{
		SaveAs();
		return;
	}

	int type = wxBITMAP_TYPE_XPM;
	wxFileName wfn(mFilename);
	
	if (wfn.GetExt().CmpNoCase("bmp")==0)
		type = wxBITMAP_TYPE_BMP;
	else if (wfn.GetExt().CmpNoCase("png")==0)
		type = wxBITMAP_TYPE_PNG;

	if (!mView->SaveFile( mFilename, type))
		wxMessageBox("Error saving image.", "Xpmedit", wxICON_ERROR|wxOK);

	UpdateTitlebar();
	UpdateControlStatus();
}

void AFFrame::SaveAs()
{
	wxString caption;
	caption.Printf("Save As: '%s'", mFilename != "" ? mFilename.c_str() : "untitled");
	
	wxString initial_dir = "";
	if ( mFilename != "" )
		initial_dir = wxPathOnly( mFilename );

	wxString FileName;
	wxFileName::SplitPath(mFilename, NULL, &FileName, NULL);
	
	wxFileDialog fdlg(this, caption, initial_dir, FileName, 
		"XPM Files(*.xpm)|*.xpm|BMP Files(*.bmp)|*.bmp|PNG Files(*.png)|*.png", wxSAVE | wxOVERWRITE_PROMPT );
	
	if (fdlg.ShowModal() == wxID_OK)
	{
		if (fdlg.GetPath() != "")
		{
			mFilename = fdlg.GetPath();
			Save();
			return;
		}
		else
		{
			wxMessageBox("No filename selected", "", wxOK | wxICON_ERROR);
		}
	}
		
}

bool AFFrame::CloseDocument()
{
	if ( mView->IsDirty() )
	{
		int ret = wxMessageBox("Document has been modified. Save it?", "Xpmedit", wxYES_NO | wxCANCEL | wxICON_EXCLAMATION);

		if (ret == wxYES)
		{
			Save();
			if ( mView->IsDirty() )
				return false;
		}
		else if (ret == wxCANCEL)
		{
			return false;
		}
	}
	
	/* UNLOAD DOCUMENT AND DATA */

	mView->Clear(24, 24);
	mView->RedrawAll();

	mFilename = "";
	UpdateTitlebar();
	UpdateControlStatus();

	return true;
}

void AFFrame::Exit()
{
	Close( false );
}

void AFFrame::OnDocumentCommand(wxCommandEvent &evt)
{
	wxImage &img = mView->GetImage();
	switch(evt.GetId())
	{
	case ID_NEW:
		New();
		break;
	case ID_RESIZE:
		mView->Resize(GetNewSize(img.GetWidth(), img.GetHeight()));
		break;
	case ID_OPEN:
		Open();
		break;
	case ID_SAVE:
		Save();
		break;
	case ID_SAVEAS:
		SaveAs();
		break;
	case ID_CLOSEFILE:
		CloseDocument();
	default:
		break;
	}
}


void AFFrame::OnPrintCommand(wxCommandEvent &evt)
{
	switch (evt.GetId())
	{
	case ID_PRINT:{

		wxPrintDialogData pdd(*app_printData);
		wxPrinter printer( &pdd );
		AFPrintout printout( "AFedit Printout", mFilename, mView );
		
		if ( !printer.Print(this, &printout, true))
		{
			if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
				wxMessageBox("There was a printing error.  Check your settings.", "", wxOK | wxICON_ERROR);
		}
		else
		{
			(*app_printData) = printer.GetPrintDialogData().GetPrintData();
		}
	   }
		break;
		
	case ID_PRINT_SETUP:
	 {
		wxPrintDialogData pdd( *app_printData );
		wxPrintDialog dlg(this, &pdd);
		
		dlg.GetPrintDialogData().SetSetupDialog(true);
		dlg.ShowModal();		
		(*app_printData) = dlg.GetPrintDialogData().GetPrintData();
	 }
		break;
		
	case ID_PAGE_SETUP:
	 {
		(*app_pageSetupData) = *app_printData;
		
		wxPageSetupDialog dlg(this, app_pageSetupData);
		dlg.ShowModal();
		
		(*app_printData) = dlg.GetPageSetupData().GetPrintData();
		(*app_pageSetupData) = dlg.GetPageSetupData();
	 }
		break;
		
	case ID_PREVIEW:
	 {
		wxPrintDialogData pdd ( *app_printData );

		wxPrintPreview *preview = new wxPrintPreview (
				new AFPrintout("AFedit Print Preview", mFilename, mView),
				new AFPrintout("AFedit Print Preview", mFilename, mView),
				& pdd );

		preview->SetZoom(35);
				
		if (!preview->Ok())
		{
			delete preview;
			wxMessageBox("There was a problem generating the preview.", "", wxOK | wxICON_ERROR);
			return;
		}
		
		wxPreviewFrame *frame = new wxPreviewFrame(preview, this, "AFedit Print Preview",
				wxDefaultPosition, wxSize(600, 600));

		frame->Center(wxBOTH);
		frame->Initialize();
		frame->Show(true);

#ifdef __WXMSW__
		frame->SetIcon( wxIcon("aficon") );
#else
		frame->SetIcon( wxIcon( af_icon_xpm ) );
#endif
	 }
		break;
	default:
		break;
	}
}

void AFFrame::OnExit(wxCommandEvent &evt)
{
	Exit();
}
void AFFrame::OnAbout(wxCommandEvent &evt)
{
	wxString msg;
	msg.Printf("Xpmedit 1.1\n"
		"Simple XPM image editor.\n\n"
		"Copyright (c) 2001-2005 Aron Dobos\n");

	wxMessageBox(msg,"About Xpmedit", wxOK | wxICON_INFORMATION);
}

void AFFrame::OnHelp(wxCommandEvent &evt)
{
	mHelpView->Show();
}

void AFFrame::UpdateControlStatus()
{
	bool DocumentPresent = true;
	bool DocumentDirty = true;
	
	mFileMenu->Enable(ID_SAVE, DocumentPresent && DocumentDirty);
	mFileMenu->Enable(ID_SAVEAS, DocumentPresent);
	mFileMenu->Enable(ID_CLOSEFILE, DocumentPresent);

	mFileMenu->Enable(ID_PRINT, false );
	mFileMenu->Enable(ID_PRINT_SETUP, false );
	mFileMenu->Enable(ID_PAGE_SETUP, false );
	mFileMenu->Enable(ID_PREVIEW, false );

	mFileMenu->Enable(ID_RECENT_FILES, mRecentCount > 0 );
	
	mToolBar->EnableTool(ID_NEW, mFileMenu->IsEnabled(ID_NEW));
	mToolBar->EnableTool(ID_OPEN, mFileMenu->IsEnabled(ID_OPEN));
	mToolBar->EnableTool(ID_SAVE, mFileMenu->IsEnabled(ID_SAVE));
	mToolBar->EnableTool(ID_SAVEAS, mFileMenu->IsEnabled(ID_SAVEAS));

}

void AFFrame::UpdateTitlebar(bool force)
{
	static bool last = false;
	bool dirty = mView->IsDirty(); /* CHECK IF DATA IS MODIFIED */

	if (last != dirty || force)
	{
		wxString title;
		title.Printf("%s%s - Xpmedit", mFilename.length() > 0 ? mFilename.c_str() : "Untitled", dirty ? "*":"");
		SetTitle( title );
		last = dirty;
	}
}

/************* AFPrintout for printing support ***************/

AFPrintout::AFPrintout(const char *title, const wxString &filename, AFView *view)
	: wxPrintout( title )
{
	mFilename = filename;
	mView = view;
}

bool AFPrintout::OnPrintPage(int page)
{
	wxDC *dc = GetDC();
	
	if (dc)
	{
		Render(dc);
		return true;
	}
	
	return false;
}

bool AFPrintout::OnBeginDocument(int startPage, int endPage)
{
	if (!wxPrintout::OnBeginDocument(startPage, endPage))
		return false;
	
	return true;
}

void AFPrintout::GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo)
{
	*minPage = 1;
	*maxPage = 1;
	*selPageFrom = 1;
	*selPageTo = 1;
}

bool AFPrintout::HasPage(int page)
{
	return (page == 1);
}

void AFPrintout::Render(wxDC *dc)
{	
	if (!mView)
		return;
	
	ViewInfo vi;
	mView->GetViewInfo(&vi);

	CalculateScale();

	ViewInfo viprint = vi;

	viprint.psz = dPrintScale;
	viprint.pixwidth = dPrintWidth;
	viprint.pixheight = dPrintHeight;
	
	mView->SetViewInfo(viprint, false);
	mView->SetPrinting( true );

	dc->SetDeviceOrigin(dDeviceOriginX, dDeviceOriginY);

	mView->Draw( *dc );

    wxDateTime now = wxDateTime::Now();
	wxString text;	
	text.Printf("%s  %s ", mFilename == "" ? "untitled" : mFilename.c_str(), 
					now.Format("%a %d %b %Y  %H:%M:%S").c_str() );


	wxFont font(dPrintHeight*0.015, wxSWISS, wxNORMAL, wxNORMAL, false);
	dc->SetFont( font );
	dc->SetTextForeground(*wxBLACK);
	dc->DrawText( text, 0, dPrintHeight);

	mView->SetPrinting( false );
	mView->SetViewInfo(vi, false);

}
	
void AFPrintout::CalculateScale()
{
	// Calculate print scale factors

/*  ---- SAMPLE CODE FROM PLEDIT 2 ----
	int nResX, nResY, nMinRes;
	
	if (IsPreview())
		GetDC()->GetSize(&nResX, &nResY);
	else
		GetPageSizePixels(&nResX, &nResY);


	if (nResX < nResY)
		nMinRes = nResX;
	else
		nMinRes = nResY;

	nMinRes *= 0.80;
	
	int minx, miny, maxx, maxy;

	mView->GetExtents(minx, miny, maxx, maxy);

	int centx, centy;
	double fminx, fmaxx, fminy, fmaxy, fdiffx, fdiffy, fratiodiff;

	int width, height;
	width = height = nMinRes;
		
	centx = (maxx + minx) / 2;
	centy = (maxy + miny) / 2;
	
	mView->VertexToCoord(minx, maxx, &fminx, &fmaxx);
	mView->VertexToCoord(miny, maxy, &fminy, &fmaxy);
	
	fminx = minx / 1000.0f;
	fmaxx = maxx / 1000.0f;
	fminy = miny / 1000.0f;
	fmaxy = maxy / 1000.0f;
	
	fdiffx = abs(fmaxx - fminx);
	fdiffy = abs(fmaxy - fminy);
	
	fratiodiff = fdiffx;
	if (fratiodiff < fdiffy)
		fratiodiff = fdiffy;
		
	dPrintScale = (width / fratiodiff)*0.9;
	
	dPrintOffsetX = ((float)width / 2) - ((float)centx / 1000.0f * dPrintScale) ;
	dPrintOffsetY = ((float)height / 2) + ((float)centy / 1000.0f * dPrintScale) ;
	dPrintWidth = 0.8*nResX;
	dPrintHeight = 0.8*nResY;

	dDeviceOriginX = (nResX - dPrintWidth)/2;
	dDeviceOriginY = (nResY - dPrintHeight)/2;
	*/

	dPrintScale = 1;
	dPrintOffsetX = dPrintOffsetY = 1;
	dPrintWidth = dPrintHeight = 100;
	dDeviceOriginX = dDeviceOriginY = 10;
}
