#ifndef AFWX_H
#define AFWX_H


extern wxConfig *app_config;


class AFApp : public wxApp
{
public:
	AFApp();
	bool OnInit();
	int OnExit();
	
};

DECLARE_APP(AFApp)

#define MAX_RECENT 9

class AFView;
class AFHelp;


#define MAXWIDTH 500
#define MAXHEIGHT 500

class AFFrame : public wxFrame
{
public:
	AFFrame();
	virtual ~AFFrame();
	
	void New();
	void Load(const wxString &fn);
	void Open();
	void Save();
	void SaveAs();
	bool CloseDocument();
	void ShowHelp();
	void Exit();
	
	void UpdateControlStatus();
	void UpdateTitlebar(bool force=true);

	wxSize GetNewSize(int w=24, int h=24);
private:	
	
	void OnDocumentCommand(wxCommandEvent &evt);
	
	void OnEditCommand(wxCommandEvent &evt);
	void OnUpdateEditMenuUI(wxUpdateUIEvent &evt);

	void OnPrintCommand(wxCommandEvent &evt);
	void OnRecent(wxCommandEvent &evt);
	void OnExit(wxCommandEvent &evt);

	void OnAbout(wxCommandEvent &evt);
	void OnHelp(wxCommandEvent &evt);
	
	void AddRecent(const wxString &fn);
	void RemoveRecent(const wxString &fn);
	void UpdateRecentMenu();

	void OnCloseFrame(wxCloseEvent &evt);
	
	wxMenu *mFileMenu, *mRecentMenu;

	wxToolBar *mToolBar;

	wxString mFilename;
	AFView *mView;
	AFHelp *mHelpView;
	
	int mRecentCount;
	wxString mRecentFiles[MAX_RECENT];

	DECLARE_EVENT_TABLE()
};

class AFPrintout : public wxPrintout
{
public:
	AFPrintout(const char *title, const wxString &filename, AFView *view);
	
	bool OnPrintPage(int page);
	bool HasPage(int page);
	bool OnBeginDocument(int startPage, int endPage);
	void GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *setPageTo);
	void CalculateScale();
	void Render(wxDC *dc);
	
private:
	AFView *mView;
	double dPrintOffsetX, dPrintOffsetY;
	double dPrintScale;
	int dPrintWidth, dPrintHeight;
	int dDeviceOriginX, dDeviceOriginY;
	wxString mFilename;
};

#endif
